# OBUSPA - 3.0 Integration notes and observation

`OB-USP-A` maintains a in-memory list of datamodel objects and it usages a sqlite3 database to store the values of objects implemented and maintained by obsupa core i.e. "Device.LocalAgnet.", to get vendor side of datamodel objects, it provides mechanism to integrate datamodel objects/parameters supported by third party application using vendor APIs. To integrate vendor side of datamodel object, it is required to first do the registration of the objects, which adds a node in `obuspa` core in-memory object list. Below are the APIs used in this integration with there meaning:

- API to create a group and register handlers to do CRUD operation on that group

```c
USP_REGISTER_GroupedObject(GROUP_ID = 0, schema_path = "Device.", is_writable = false);
USP_REGISTER_GroupVendorHooks(GROUP_ID = 0, Get_cb, Set_cb, Add_cb, Del_cb);
```

- API to register/associate leaf parameters with an existing group

```c
USP_REGISTER_GroupedVendorParam_ReadWrite(group_id, spath, type);
USP_REGISTER_GroupedVendorParam_ReadOnly(group_id, spath, type);
```

- API to register a parameter as constant

```c
USP_REGISTER_Param_Constant(spath, val, dm_type);
```

- API to register NumberOfEntries parameter

```
USP_REGISTER_Param_NumEntries(spath, param);
```

- API to register unique keys along with multi-instance object

```
USP_REGISTER_Object_UniqueKey(spath, unique_keys, num_keys);
```

- API to register operate 

```c
USP_REGISTER_SyncOperation(path, uspd_operate_sync);
USP_REGISTER_AsyncOperation(path, uspd_operate_async, NULL);
USP_REGISTER_OperationArguments(spath, in, in_nr, out, out_nr);
```

- Vendor hooks to extend or override core functionality

  - List of available vendor hooks

    ```c
    dm_vendor_get_agent_serial_number_cb_t  get_agent_serial_number_cb;
    dm_vendor_get_agent_endpoint_id_cb_t    get_agent_endpoint_id_cb;
    dm_vendor_reboot_cb_t                   reboot_cb;
    dm_vendor_factory_reset_cb_t            factory_reset_cb;
    // Vendor hooks associated with vendor database transactions
    dm_vendor_start_trans_cb_t              start_trans_cb;
    dm_vendor_commit_trans_cb_t             commit_trans_cb;
    dm_vendor_abort_trans_cb_t              abort_trans_cb;
    // Vendor hooks associated with certificates and controller trust
    register_controller_trust_cb_t          register_controller_trust_cb;
    is_system_time_reliable_cb_t            is_system_time_reliable_cb;
    get_trust_store_cb_t                    get_trust_store_cb;
    get_agent_cert_cb_t                     get_agent_cert_cb;
    // Miscellaneous vendor hooks
    #ifndef REMOVE_DEVICE_INFO
        get_active_software_version_cb_t        get_active_software_version_cb;
        get_hardware_version_cb_t               get_hardware_version_cb;
    #endif
    dm_vendor_get_mtp_password_cb_t         get_mtp_password_cb;
    load_agent_cert_cb_t                    load_agent_cert_cb;
    log_message_cb_t                        log_message_cb;
    ```

  - List of implemented vendor_hook callbacks

    ```c
    callbacks.reboot_cb = reboot;
    callbacks.factory_reset_cb = factory_reset;
    callbacks.start_trans_cb = uspd_tran_start;
    callbacks.commit_trans_cb = uspd_tran_commit;
    callbacks.abort_trans_cb = uspd_tran_abort;
    return USP_REGISTER_CoreVendorHooks(&callbacks);
    ```

- APIs defined in obuspa-2, but not used in this integration

  ```c
  // Registers a multi-instance object
  USP_REGISTER_Object(spath, validate_add_cb, add_cb, notify_add_cb, validate_del_cb, del_cb, notify_del_cb);
  
  // Register a object as sqlite db parameter
  USP_REGISTER_DBParam_Alias(spath, notify_set_cb);
  USP_REGISTER_DBParam_ReadWrite(spath, value, validator_cb, notify_set_cb, type_flags);
  USP_REGISTER_DBParam_ReadOnly(spath, value, type_flags);
  
  ```

  


### Vendor Init

This segment is called by `obuspa` daemon as well as by `obuspa -c` command line tool. It is required by `obuspa` core data structure to register the available datamodel schema (datamodel objects and parameters) before it starts.

- Creates global ubus ctx for uspd interaction
  - `g_ubus_ctx = ubus_connect(NULL);`
- Local kv structure to hold/cache mandatory parameters
- Do a group get call to get all the mandatory parameters and store it in kv_vector to be used in later stage

```c
USP_ARG_Add(kv, "Device.DeviceInfo.SoftwareVersion", "");
USP_ARG_Add(kv, "Device.DeviceInfo.ProductClass", "");
USP_ARG_Add(kv, "Device.DeviceInfo.Manufacturer", "");
USP_ARG_Add(kv, "Device.DeviceInfo.ModelName", "");
USP_ARG_Add(kv, "Device.DeviceInfo.HardwareVersion", "");
USP_ARG_Add(kv, "Device.DeviceInfo.ManufacturerOUI", "");
USP_ARG_Add(kv, "Device.DeviceInfo.SerialNumber", "");
```

- Initialise a global str_vector `STR_VECTOR_Init(&g_group_vec);` to store names of all registered groups

- Register `Device.` as base group with group_id 0

```c
STR_VECTOR_Add(&g_group_vec, "Device.");
USP_REGISTER_GroupedObject(ROOT_GROUP_ID, "Device.", false);
USP_REGISTER_GroupVendorHooks(ROOT_GROUP_ID, GetMyParams, SetMyParams, AddMyObject, DelMyObject);

```

- Get and register schema from `uspd` using `dump_schema` method

- For each parameter in parameters_list of `dump_schema`

  - STORE the unique_keys list to be processed later on

  - SKIP_registration (if the schema starts with 'Device.BulkData.' or 'Device.Security.')

  - Register a parameter as constants (if schema is a mandatory parameter) `USP_REGISTER_Param_Constant`

  - If schema dmtype is Object

    - if (group already exists with top data-model schema) then register the object schema with existing group `USP_REGISTER_GroupedObject`
    - else (Create a new group and add group handlers also register refresh_instances with that schema)

    ```c
    USP_REGISTER_GroupedObject(group_id, spath, rw_perm);
    USP_REGISTER_GroupVendorHooks(group_id, GetMyParams, SetMyParams, AddMyObject, DelMyObject);
    USP_REGISTER_Object_RefreshInstances(spath, refresh_instances);
    
    ```

    

  - If schema dmtype is non-object

    - Register the parameter as `USP_REGISTER_Param_NumEntries` if schema ends with `NumberOfEntries`
    - Get group number to which this schema belong and register leaf data-model object with that group 

    ```c
    USP_REGISTER_GroupedVendorParam_ReadWrite(group, spath, type);
    USP_REGISTER_GroupedVendorParam_ReadOnly(group, spath, type);
    ```

    

  - Register uniq key parameters `USP_REGISTER_Object_UniqueKey(spath, keys, num_keys);`

- Register all the supported operate commands get from `uspd` `list_operate` command

- for each entry in list_operate list 

  - Skip registration of 'Device.Reboot()' and 'Device.FactoryReset()' operations, as they will be attached to vendor hooks later on
  - Register rest of the operate commands using

  ```c
  USP_REGISTER_SyncOperation
  USP_REGISTER_AsyncOperation
  USP_REGISTER_OperationArguments
  ```

- Register Vendor hooks

```c
    callbacks.reboot_cb = reboot;
	callbacks.factory_reset_cb = factory_reset;
	callbacks.start_trans_cb = uspd_tran_start;
	callbacks.commit_trans_cb = uspd_tran_commit;
	callbacks.abort_trans_cb = uspd_tran_abort;
	return USP_REGISTER_CoreVendorHooks(&callbacks);
```



### Vendor Start

- Create a socketpair for datamodel thread and ubus thread communication
- Start a ubus thread for communication with uspd over ubus, this thread waits on `uloop_run` to get async call backs
- socket pair is registered with uloop using `uloop_fd_add` to get messages from core-dm-thread

```c
int vendor_uspd_start(void)
{
	int fault = USP_ERR_OK;

	fault = vendor_exec_init();
	if (fault == USP_ERR_OK)
		fault = OS_UTILS_CreateThread(start_ubus_thread, NULL);

	return fault;
}
```


### Vendor Stop

- Delete global structures created so far

```c
int vendor_uspd_stop(void)
{
	STR_VECTOR_Destroy(&g_group_vec);
	ubus_free(g_ubus_ctx);
	g_ubus_ctx = NULL;
	return USP_ERR_OK;
}
```

### Optimisations done around the APIs

- Formation of groups for group api registration
  - New groups registered for each top level multi-instance object, i.e a new group created for`Device.WiFi.AccessPoint.{i}.` and then `Device.WiFi.AccessPoint.{i}.AssociatedDevice.{i}.` is registered as a part of the same group.
  - A refresh instance is required as well for the multi-instance objects, which is also created on top level multi-instance object
  - Group get call is further sub-divided into two parts
    - when number of parameters less than per defined MIN_NUM_TO_GROUP value
      - `getm_value` is used to get only required parameters
    - when it's more than MIN_NUM_TO_GROUP
      - `get` called with the group path
    - It means, if `get` operation is called on `Device.WiFi.SSID.1.SSID`, the callback function gets the group id (which further translate to group `Device.WiFi.SSID.`), along with a kv tuple list(in this case this shall have `Device.WiFi.SSID.1.SSID`) of the parameters which is actually requested, Above optimisation is done to save some time, as `get` of `Device.WiFi.SSID.1.SSID` shall take significantly less time than get and find of `Device.WiFi.SSID.`
- Caching of `Device.` output to optimise further calls to uspd for sub-object list
  - When a get operation is performed on any path, `obuspa` first needs to update the multi-instance object instances and then do the actual get, to get the reliable data from 3rd party applications
  - Once, the instance update is done, it then need to perform get for each and every group which has dm parameters registered with required path
  - So, when `get` is called on `Device.`
    - it first does the refresh instances for all the multi-instance objects. ex: `Device.WiFi.SSID.`, `Device.Users.User.`, `Device.WiFi.Radio.` etc
    - and then do the get call on each group, ex: `Device.`, `Device.WiFi.SSID.`, `Device.Users.User.`, `Device.WiFi.Radio` etc
    - Since, get of `Device.` already has all the parameters required for subsequent get calls, so this information is cached and for the subsequent call, in palace of getting the valued from `uspd`, this cached information is used to fill the get_cb.

### Notes

* Cache expiry timer is hard-coded to 30 sec
* Timeout to get a response from `uspd` is hard-coded to 10 sec
* Minimum number of objects required to do a group call is 10
* Default ubus socket is used for the obsupa and uspd communication
* Get operation on `Device.`
    * takes around 2 - 6 sec with current implementation
    * without optimisation it took around 20 - 30 sec
    * without group calls it took around 60 - 90 sec
